package at.jku.ssw.util;

import java.util.Arrays;

import at.jku.students.map.HashMapQuadraticProbing;

/** Prime number calculator. */
public class PrimeNumbers {

	private static boolean[] primes = new boolean[0];

	/**
	 * Calculates the next prime number which is greater or equal than number.
	 */
	public static int nextPrime(int number) {
		while (!isPrime(number)) {
			number++;
		}
		return number;
	}

	/**
	 * Tests if the specified number is prime.
	 */
	public static boolean isPrime(int n) {
		if (n >= primes.length) {
			calculate(2 * n);
		}
		return primes[n];
	}

	private static void calculate(int n) {
		primes = new boolean[n];
		Arrays.fill(primes, 0, n, true);
		for (int i = 2; i < n; i++) {
			if (primes[i]) {
				for (int j = i * 2; j < n; j += i) {
					primes[j] = false;
				}
			}
		}
	}

	/**
	 * Calculates an appropriate table size for {@link HashMapQuadraticProbing}.
	 * For the returned value <code>val</code> the following holds:
	 * <ul>
	 * <li><code>val >= approx</code>
	 * <li><code>val</code> a Prime number
	 * <li>there is an <code>x</code> so that <code>ret = 4 * x - 3</code>
	 * </ul>
	 */
	public static int calculateTabSize(int approx) {
		int x = (int) Math.ceil((approx - 3) / 4d);
		while (!isPrime(3 + 4 * x)) {
			x++;
		}
		return 3 + 4 * x;
	}

}